<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

use Migrations\AbstractSeed;

/**
 * Class InstallSeed
 *
 * Created:
 * oitc bake seed -p EventcorrelationModule --table cronjobs --data Install
 *
 * Apply:
 * oitc migrations seed -p EventcorrelationModule
 */
class InstallSeed extends AbstractSeed {

    /**
     * Run Method.
     *
     * Write your database seeder using this method.
     *
     * More information on writing seeds is available here:
     * http://docs.phinx.org/en/latest/seeding.html
     *
     * @return void
     */
    public function run(): void {
        //Commands
        $CommandsMigrationTable = $this->table('commands');
        $data = [
            [
                'name'         => 'check-evc-host',
                'command_line' => '/opt/openitc/frontend/bin/cake EventcorrelationModule.evc_plugin --uuid "$HOSTNAME$"',
                'command_type' => HOSTCHECK_COMMAND,
                'human_args'   => null,
                'uuid'         => '97e51c72-91c5-4f37-98fc-fd1219705e74',
                'description'  => 'Active check to calculate the state of a given event correlation.',
            ],
            [
                'name'         => 'check-evc-service',
                'command_line' => '$USER1$/check_dummy 3',
                'command_type' => CHECK_COMMAND,
                'human_args'   => null,
                'uuid'         => 'e166bc6f-2651-4bca-bf07-cdb0cf0f584f',
                'description'  => 'Dummy check required for event correlation.',
            ]
        ];

        //Check if records exists
        foreach ($data as $index => $record) {
            $QueryBuilder = $this->getAdapter()->getQueryBuilder();

            $stm = $QueryBuilder->select('*')
                ->from($CommandsMigrationTable->getName())
                ->where([
                    'uuid' => $record['uuid']
                ])
                ->execute();
            $result = $stm->fetchAll();

            if (empty($result)) {
                $CommandsMigrationTable->insert($record)->save();
            }
        }

        //Host templates
        $HosttemplatesMigrationTable = $this->table('hosttemplates');
        $hosttemplats = [
            [
                'uuid'                              => \itnovum\openITCOCKPIT\Core\UUID::v4(),
                'name'                              => 'evc-default-host',
                'description'                       => 'Monitores and correlation given services for the event correlation module.',
                'hosttemplatetype_id'               => EVK_HOST,
                'command_id'                        => '97e51c72-91c5-4f37-98fc-fd1219705e74',
                'check_command_args'                => '',
                'eventhandler_command_id'           => '0',
                'timeperiod_id'                     => '0',
                'check_interval'                    => '60',
                'retry_interval'                    => '60',
                'max_check_attempts'                => '3',
                'first_notification_delay'          => '0',
                'notification_interval'             => '7200',
                'notify_on_down'                    => '1',
                'notify_on_unreachable'             => '1',
                'notify_on_recovery'                => '1',
                'notify_on_flapping'                => '0',
                'notify_on_downtime'                => '0',
                'flap_detection_enabled'            => '0',
                'flap_detection_on_up'              => '0',
                'flap_detection_on_down'            => '0',
                'flap_detection_on_unreachable'     => '0',
                'low_flap_threshold'                => '0',
                'high_flap_threshold'               => '0',
                'process_performance_data'          => '0',
                'freshness_checks_enabled'          => '0',
                'freshness_threshold'               => '0',
                'passive_checks_enabled'            => '0',
                'event_handler_enabled'             => '0',
                'active_checks_enabled'             => '1',
                'retain_status_information'         => '0',
                'retain_nonstatus_information'      => '0',
                'notifications_enabled'             => '1',
                'notes'                             => '',
                'priority'                          => '1',
                'check_period_id'                   => '1',
                'notify_period_id'                  => '1',
                'tags'                              => '',
                'container_id'                      => ROOT_CONTAINER,
                'host_url'                          => '',
                'hosttemplatecommandargumentvalues' => [],
                'customvariables'                   => [],
                'hostgroups'                        => [],
                'contactgroups'                     => [],
                'contacts'                          => [
                    '_ids' => [
                        (int)0 => '1'
                    ]
                ]
            ]
        ];

        //Check if records exists
        foreach ($hosttemplats as $index => $record) {
            /** @var \App\Model\Table\HosttemplatesTable $HosttemplatesTable */
            $HosttemplatesTable = \Cake\ORM\TableRegistry::getTableLocator()->get('Hosttemplates');

            $QueryBuilder = $this->getAdapter()->getQueryBuilder();

            $stm = $QueryBuilder->select('*')
                ->from($HosttemplatesMigrationTable->getName())
                ->where([
                    'name' => $record['name']
                ])
                ->execute();
            $result = $stm->fetchAll();

            if (empty($result)) {
                $record['command_id'] = $this->getCommandIdByCommandUuid($record['command_id']);
                $entity = $HosttemplatesTable->newEntity($record);
                $HosttemplatesTable->save($entity);
            }
        }

        //Service templates
        $ServicetemplatesMigrationTable = $this->table('servicetemplates');
        $servicetemplats = [
            [
                'uuid'                                      => \itnovum\openITCOCKPIT\Core\UUID::v4(),
                'template_name'                             => 'evc-default-service',
                'name'                                      => 'evc-default-service',
                'container_id'                              => ROOT_CONTAINER,
                'servicetemplatetype_id'                    => EVK_SERVICE,
                'check_period_id'                           => '1',
                'notify_period_id'                          => '1',
                'description'                               => 'Virtual service to store the result of an event correlation.',
                'command_id'                                => 'e166bc6f-2651-4bca-bf07-cdb0cf0f584f',
                'check_command_args'                        => '',
                'checkcommand_info'                         => '',
                'eventhandler_command_id'                   => '0',
                'timeperiod_id'                             => null,
                'check_interval'                            => '300',
                'retry_interval'                            => '60',
                'max_check_attempts'                        => '3',
                'first_notification_delay'                  => '0',
                'notification_interval'                     => '7200',
                'notify_on_warning'                         => '1',
                'notify_on_unknown'                         => '1',
                'notify_on_critical'                        => '1',
                'notify_on_recovery'                        => '1',
                'notify_on_flapping'                        => '0',
                'notify_on_downtime'                        => '0',
                'flap_detection_enabled'                    => '0',
                'flap_detection_on_ok'                      => '0',
                'flap_detection_on_warning'                 => '0',
                'flap_detection_on_unknown'                 => '0',
                'flap_detection_on_critical'                => '0',
                'low_flap_threshold'                        => '0',
                'high_flap_threshold'                       => '0',
                'process_performance_data'                  => '1',
                'freshness_checks_enabled'                  => '0',
                'freshness_threshold'                       => null,
                'passive_checks_enabled'                    => '0',
                'event_handler_enabled'                     => '0',
                'active_checks_enabled'                     => '0',
                'retain_status_information'                 => '0',
                'retain_nonstatus_information'              => '0',
                'notifications_enabled'                     => '1',
                'notes'                                     => '',
                'priority'                                  => '1',
                'tags'                                      => '',
                'service_url'                               => '',
                'is_volatile'                               => '0',
                'check_freshness'                           => '0',
                'servicetemplateeventcommandargumentvalues' => [],
                'servicetemplatecommandargumentvalues'      => [],
                'customvariables'                           => [],
                'servicegroups'                             => [],
                'contactgroups'                             => [],
                'contacts'                                  => []
            ]
        ];

        //Check if records exists
        foreach ($servicetemplats as $index => $record) {
            /** @var \App\Model\Table\ServicetemplatesTable $ServicetemplatesTable */
            $ServicetemplatesTable = \Cake\ORM\TableRegistry::getTableLocator()->get('Servicetemplates');

            $QueryBuilder = $this->getAdapter()->getQueryBuilder();

            $stm = $QueryBuilder->select('*')
                ->from($ServicetemplatesMigrationTable->getName())
                ->where([
                    'template_name' => $record['template_name']
                ])
                ->execute();
            $result = $stm->fetchAll();

            if (empty($result)) {
                $record['command_id'] = $this->getCommandIdByCommandUuid($record['command_id']);
                $entity = $ServicetemplatesTable->newEntity($record);
                $ServicetemplatesTable->save($entity);
            }
        }

    }

    /**
     * @param string $uuid
     * @return int
     */
    private function getCommandIdByCommandUuid(string $uuid) {
        $CommandsMigrationTable = $this->table('commands');

        $QueryBuilder = $this->getAdapter()->getQueryBuilder();

        $stm = $QueryBuilder->select('*')
            ->from($CommandsMigrationTable->getName())
            ->where([
                'uuid' => $uuid
            ])
            ->execute();
        $result = $stm->fetchAll('assoc');

        if (empty($result)) {
            throw new \Cake\Datasource\Exception\RecordNotFoundException('No command for given UUID found!');
        }

        return (int)$result[0]['id'];
    }
}
