<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace EventcorrelationModule\Command;

use App\itnovum\openITCOCKPIT\Monitoring\Naemon\ExternalCommands;
use App\Lib\Interfaces\ServicestatusTableInterface;
use App\Model\Table\HostsTable;
use Cake\Command\Command;
use Cake\Console\Arguments;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\ORM\Query;
use Cake\ORM\TableRegistry;
use Cake\Utility\Hash;
use EventcorrelationModule\Model\Table\EventcorrelationSettingsTable;
use EventcorrelationModule\Model\Table\EventcorrelationsTable;
use itnovum\openITCOCKPIT\Core\DbBackend;
use itnovum\openITCOCKPIT\Core\Servicestatus;
use itnovum\openITCOCKPIT\Core\ServicestatusFields;

/**
 * Class EvcPluginCommand
 * @package EventcorrelationModule\Command
 */
class EvcPluginCommand extends Command {

    const OK = 0;
    const WARNING = 1;
    const CRITICAL = 2;
    const UNKNOWN = 3;

    /**
     * Hook method for defining this command's option parser.
     *
     * @see https://book.cakephp.org/3.0/en/console-and-shells/commands.html#defining-arguments-and-options
     *
     * @param \Cake\Console\ConsoleOptionParser $parser The parser to be defined
     * @return \Cake\Console\ConsoleOptionParser The built parser.
     */
    public function buildOptionParser(ConsoleOptionParser $parser): ConsoleOptionParser {
        $parser = parent::buildOptionParser($parser);

        $parser->addOption('uuid', [
            'help'    => 'Event correlation host uuid',
            'default' => false
        ]);

        return $parser;
    }

    /**
     * Implement this method with your command's logic.
     *
     * @param \Cake\Console\Arguments $args The command arguments.
     * @param \Cake\Console\ConsoleIo $io The console io
     * @return null|void|int The exit code or null for success
     * @throws \App\Lib\Exceptions\MissingDbBackendException
     */

    public function execute(Arguments $args, ConsoleIo $io) {
        $uuid = $args->getOption('uuid');

        if (!$uuid) {
            $io->out('Missing option --uuid');
            exit(2);
        }

        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

        $evcHost = $HostsTable->getHostByUuid($uuid);

        $DbBackend = new DbBackend();

        /** @var EventcorrelationsTable $EventcorrelationsTable */
        $EventcorrelationsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.Eventcorrelations');
        /** @var EventcorrelationSettingsTable $EventcorrelationSettingsTable */
        $EventcorrelationSettingsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.EventcorrelationSettings');
        /** @var ServicestatusTableInterface $ServicestatusTable */
        $ServicestatusTable = $DbBackend->getServicestatusTable();

        $currentEventcorrelationSettings = $EventcorrelationSettingsTable->getCurrentEventcorrelationSettings();
        $defaultEventcorrelationSettings = $EventcorrelationSettingsTable->getDefaultEventcorrelationSettings();

        $considerStateType = ($defaultEventcorrelationSettings['EVC_CONSIDER_STATETYPE']['value'] & $currentEventcorrelationSettings->get('configuration_option'));
        $stateForDisabledService = $defaultEventcorrelationSettings['EVC_STATUS_DISABLED_SERVICE']['value'];
        if (!empty($currentEventcorrelationSettings)) {
            $stateForDisabledService = $currentEventcorrelationSettings->get('status_disabled_service');
        }
        $longOutputArray = [];
        $rootServiceState = self::UNKNOWN;
        $rootServiceFullName = null;
        $serviceName = null;

        $evcServices = $this->getEvcTree($evcHost->get('id'));

        $ExternalCommands = new ExternalCommands();

        if (!empty($evcServices)) {
            $evcServices = $this->reorganizeEvcStructure($evcServices);
            $evcServices = array_reverse($evcServices);
            $tmpResult = [];

            $ServicestatusFields = new ServicestatusFields($DbBackend);
            $ServicestatusFields
                ->currentState()
                ->isHardstate()
                ->lastStateChange()
                ->output();

            foreach ($evcServices as $layerIndex => $evcData) {
                foreach ($evcData as $evcId => $evcChilds) {
                    foreach ($evcChilds as $index => $evc) {
                        $serviceOutput = '';
                        if ($layerIndex === 0) {
                            $currentState = self::UNKNOWN;
                            if(!$evc['service']['disabled']){
                                $tmpServicestatus = $ServicestatusTable->byUuid($evc['service']['uuid'], $ServicestatusFields);
                                if (!empty($tmpServicestatus)) {
                                    $Servicestatus = new Servicestatus($tmpServicestatus['Servicestatus']);
                                    $serviceOutput = ' OUTPUT:' . $Servicestatus->getOutput();
                                    $currentState = $Servicestatus->currentState();
                                    if ($considerStateType && !$Servicestatus->isHardstate()) {
                                        $currentState = $Servicestatus->getLastHardState();
                                    }
                                }
                            }else{
                                $serviceOutput = ' OUTPUT: DISABLED';
                                $currentState = $stateForDisabledService;
                            }


                            $tmpResult[$evcId][] = (int)$currentState;
                            if (isset($evc['service']['host']['name'])) {
                                $serviceName = $evc['service']['host']['name'] . '/' . $evc['servicename'];
                                $longOutputArray[$evcId][$currentState][] = $serviceName . $serviceOutput;
                            }
                        } else {
                            $evcResult = $this->currentEvcState($evc['operator'], $tmpResult[$evc['id']], ($defaultEventcorrelationSettings['EVC_CONSIDER_STATE_COUNT']['value']
                                & $currentEventcorrelationSettings->get('configuration_option')));
                            $tmpResult[$evcId][] = $evcResult['state'];
                            $tmpServicestatus = $ServicestatusTable->byUuid($evc['service']['uuid'], $ServicestatusFields);
                            if (!empty($tmpServicestatus)) {
                                $Servicestatus = new Servicestatus($tmpServicestatus['Servicestatus']);
                                $serviceOutput = ' OUTPUT:' . $Servicestatus->getOutput();
                            }
                            if (isset($evc['service']['host']['name'])) {
                                $serviceName = $evc['service']['host']['name'] . '/' . $evc['servicename'];
                                $longOutputArray[$evcId][$evcResult['state']][] = $serviceName . $serviceOutput;
                            }
                            ksort($longOutputArray[$evc['id']]);
                            $checkData = [
                                'hostUuid'     => $evc['service']['host']['uuid'],
                                'serviceUuid'  => $evc['service']['uuid'],
                                'state'        => $evcResult['state'],
                                'comment'      => $evcResult['output'] . ' |state=' . $evcResult['state'] . ';1;2;0;3' . PHP_EOL,
                                'long_output'  => $this->getLongOutputString($longOutputArray[$evc['id']]),
                                'satellite_id' => 0 //EVC results are always from the Mastersystem
                            ];

                            if ($currentEventcorrelationSettings->get('monitoring_system') === 'statusengine') {
                                //Use the statusengine event broker to submit evc results
                                $ExternalCommands->passiveTransferServiceCheckresult($checkData);
                            } else if ($currentEventcorrelationSettings->get('monitoring_system') === 'naemon') {
                                // use Naemon Query Handler - LEGACY use Statusengine !

                                $checkData['comment'] .= $checkData['long_output'];
                                unset($checkData['long_output']);
                                unset($checkData['satellite_id']);

                                $command = "#command run ";
                                $command .= '[' . time() . '] PROCESS_SERVICE_CHECK_RESULT;' . implode(';', $checkData);
                                $command .= "\0";

                                $NagiosQH = socket_create(AF_UNIX, SOCK_STREAM, 0);
                                $connectionResult = socket_connect($NagiosQH, '/opt/openitc/nagios/var/rw/nagios.qh');
                                if ($connectionResult) {
                                    socket_set_nonblock($NagiosQH);
                                    socket_write($NagiosQH, $command);
                                    socket_close($NagiosQH);
                                }
                            } else {
                                //use nagios / naemon.cfg - LEGACY use Statusengine !
                                //No support for long output
                                if (file_exists('/opt/openitc/nagios/var/rw/nagios.cmd')) {
                                    $nagiosCmd = fopen('/opt/openitc/nagios/var/rw/nagios.cmd', 'w+');
                                    $command = '[' . time() . '] PROCESS_SERVICE_CHECK_RESULT;' . $checkData['hostUuid'] . ';' . $checkData['serviceUuid'] . ';' . $checkData['state'] . ';' . $checkData['comment'];
                                    fwrite($nagiosCmd, $command);
                                    fclose($nagiosCmd);
                                }
                            }
                            $rootServiceState = $evcResult['state'];
                            $rootServiceFullName = $serviceName;

                        }
                    }
                }
            }
            if (($defaultEventcorrelationSettings['EVC_REFLECT_CRITICAL_STATE']['value'] & $currentEventcorrelationSettings->get('configuration_option'))
                && ($rootServiceState === self::CRITICAL)) {
                $io->out('EVC Check - ' . $rootServiceFullName . ' is critical!');
                exit(1);
            }
        }
        $io->out('Event correlation processed successfully');
        exit(0);
    }

    /**
     * @param int $evcHostId
     * @return array
     */
    public function getEvcTree($evcHostId) {
        /** @var EventcorrelationsTable $EventcorrelationsTable */
        $EventcorrelationsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.Eventcorrelations');

        $query = $EventcorrelationsTable->find('threaded')
            ->contain([
                'Services' => function (Query $query) {
                    $query->disableAutoFields()
                        ->select([
                            'Services.name',
                            'Services.uuid',
                            'Services.disabled'
                        ])
                        ->contain([
                            'Hosts'            => function (Query $query) {
                                $query->disableAutoFields()
                                    ->select([
                                        'Hosts.uuid',
                                        'Hosts.name',
                                    ]);
                                return $query;
                            },
                            'Servicetemplates' => function (Query $query) {
                                $query->disableAutoFields()
                                    ->select([
                                        'Servicetemplates.id',
                                        'Servicetemplates.name'
                                    ]);
                                return $query;
                            }
                        ]);
                    return $query;
                },
            ])
            ->where([
                'Eventcorrelations.host_id' => $evcHostId
            ])
            ->disableHydration();

        if (empty($query)) {
            return [];
        }

        return $query->toArray();
    }

    public function reorganizeEvcStructure($evcAsNest) {
        $result = [];
        function flattenElement($children, $layer, &$result) {
            foreach ($children as $element) {
                $data = $element;

                $servicename = $element['service']['name'];
                if ($servicename === null || $servicename === '') {
                    $servicename = $element['service']['servicetemplate']['name'];
                }

                $data['servicename'] = $servicename;
                $result[$layer][$element['parent_id']][] = $data;
            }
            foreach ($children as $element) {
                flattenElement($element['children'], $layer + 1, $result);
            }
        }

        flattenElement($evcAsNest, 1, $result);

        return $result;
    }

    public function currentEvcState($operator, $stateArray = [], $considerStateCount = false) {
        $evcState = [
            'state'  => self::UNKNOWN,
            'output' => __('0 services are OK'),
        ];
        $stateCounter = sizeof($stateArray);
        $stateSummary = array_count_values(Hash::filter($stateArray));
        if (empty($stateSummary)) {
            return $evcState;
        }
        $operatorDetails = (strpos($operator, 'min') === 0) ? array_combine(
            ['operator', 'operatorModifier'],
            array_filter(preg_split('/(\d+)/', $operator, -1, PREG_SPLIT_DELIM_CAPTURE))) : ['operator' => $operator];
        switch ($operatorDetails['operator']) {
            case 'and':
                if (isset($stateSummary[0]) && ($stateSummary[0] === $stateCounter)) {
                    $evcState['state'] = self::OK;

                } else {
                    $evcState['state'] = ($considerStateCount) ? $this->checkResultConsiderStatePriority($stateSummary) : max(array_keys($stateSummary));
                }
                break;
            case 'or':
                if (isset($stateSummary[0])) {
                    $evcState['state'] = self::OK;
                } else {
                    $evcState['state'] = ($considerStateCount) ? $this->checkResultConsiderStatePriority($stateSummary) : max(array_keys($stateSummary));
                }
                break;
            case 'min':
                if (isset($stateSummary[0]) && ($stateSummary[0] >= $operatorDetails['operatorModifier'])) {
                    $evcState['state'] = self::OK;
                } else {
                    $evcState['state'] = ($considerStateCount) ? $this->checkResultConsiderStatePriority($stateSummary) : max(array_keys($stateSummary));
                }
                break;
            case 'eq':
                if ($stateCounter === 1 && !empty($stateSummary)) {
                    $evcState['state'] = $stateArray[0];
                }
                break;
        }
        $evcState['output'] = __('{0} - {1} services ({2} from {3} services are OK)', strtoupper($operator),
            (isset($stateSummary[0])) ? $stateSummary[0] : 0,
            (isset($stateSummary[0])) ? $stateSummary[0] : 0,
            $stateCounter
        );

        return $evcState;
    }

    public function getLongOutputString($stateSummary = []) {
        $longoutput = "";
        $stateArray = ['OK', 'WARNING', 'CRITICAL', 'UNKNOWN'];
        foreach ($stateSummary as $state => $values) {
            $longoutput .= $stateArray[$state] . ':' . PHP_EOL; //new v4
            foreach ($values as $line) { //new v4
                $longoutput .= '    ' . $line . PHP_EOL; // new v4
            } //new v4

            $longoutput .= PHP_EOL;


            //Old V3 long output
            //$longoutput .= PHP_EOL. $stateArray[$state] . ": " . implode(", ", $values);
        }

        return $longoutput;
    }

    public function checkResultConsiderStatePriority($stateSummary = []) {
        $state = self::UNKNOWN;
        if (empty($stateSummary)) {
            return $state;
        }
        if ((isset($stateSummary[self::CRITICAL], $stateSummary[self::UNKNOWN]) && $stateSummary[self::CRITICAL] >= $stateSummary[self::UNKNOWN]) || !isset($stateSummary[self::UNKNOWN])) {
            $state = self::CRITICAL;
        } else if (isset($stateSummary[self::UNKNOWN], $stateSummary[self::WARNING]) && $stateSummary[self::UNKNOWN] > $stateSummary[self::WARNING]) {
            $state = self::UNKNOWN;
        } else if (isset($stateSummary[self::WARNING], $stateSummary[self::UNKNOWN]) && !isset($stateSummary[self::CRITICAL]) && $stateSummary[self::WARNING] >= $stateSummary[self::UNKNOWN]) {
            $state = self::WARNING;
        }

        return $state;
    }
}
